# Kado
[![pipeline status](https://git.nullivex.com/kado/kado/badges/4.x/pipeline.svg)](https://git.nullivex.com/kado/kado/commits/4.x)
[![Build Status](https://travis-ci.org/KadoOrg/kado.svg?branch=master)](https://travis-ci.org/KadoOrg/kado)
[![npm version](https://badge.fury.io/js/kado.svg)](https://badge.fury.io/js/kado)
[![Join the chat at https://gitter.im/KadoOrg/Lobby](https://badges.gitter.im/Join%20Chat.svg)](https://gitter.im/KadoOrg/Lobby?utm_source=badge&utm_medium=badge&utm_campaign=pr-badge&utm_content=badge)

High Quality JavaScript Libraries based on ES6+

## Motivations

Kado provides solutions by combining all the common gaps in JavaScript into a
consistent, well documented, thoroughly tested collection of libraries that have
zero external dependencies.

## Why use Kado?

* Libraries for most common application needs.
* 0 external dependencies. The buck stops here!
* LGPL 3.0 License. Use Kado where you need it.
* Thoroughly tested, continuously integrated, actively developed.
* Make your own framework in just a few minutes!
* Code is peer reviewed, openly developed, openly licensed.
* Used by applications serving billions of page views.
* Assertion, validation, and test running built in.
* More! Check the library list below!

## Kado Library List

This is an exhaustive lise of all the JavaScript libraries provided with Kado.
Each library has a link to its documentation as well as a short description of
what the library provides.

* [Application](./doc/Application.md) - Create a new application containing most
Kado features.
* [Assert](./doc/Assert.md) - Make assertions on input or tests.
* [Asset](./doc/Asset.md) - Store, filter, and query static application
files.
* [ChildProcess](./doc/ChildProcess.md) - Child process library adding convenience and
functionality to the core child_process functions.
* [Cluster](./doc/Cluster.md) - Cluster library for constructing and
operating process clusters.
* [Command](./doc/Command.md) - Build CLI applications with ease.
* [CommandServer](./doc/CommandServer.md) - Execute CLI applications like a
web server.
* [Connect](./doc/Connect.md) - Framework for housing external resource
connections.
* [ConnectEngine](./doc/ConnectEngine.md) - Interface for creating an engine
to be used with a Connect system.
* [Cron](./doc/Cron.md) - Execute functions on a schedule similar to UNIX
cron jobs.
* [Database](./doc/Database.md) - Connect system made for Databases.
* [Email](./doc/Email.md) - Connect system made for Email.
* [Event](./doc/Event.md) - Create, track and handle application events with
log levels.
* [FileSystem](./doc/FileSystem.md) - Consistent API for use with File System methods.
* [Format](./doc/Format.md) - Commonly used String, Number, and Date format
methods.
* [GetOpt](./doc/GetOpt.md) - Parse command line string input into an
object.
* [History](./doc/History.md) - Track user navigation history throughout
a session.
* [HyperText](./doc/HyperText.md) - Connect system made for HTTP servers.
* [Language](./doc/Language.md) - Internationalization helpers including
loading, parsing, and displaying languages.
* [Library](./doc/Library.md) - Dynamic library loader.
* [Lifecycle](./doc/Lifecycle.md) - Start and stop systems with events.
* [Log](./doc/Log.md) - Connect system made for Logs.
* [Mapper](./doc/Mapper.md) - ECMA Map functionality on Objects.
* [Message](./doc/Message.md) - Create, track and handle messages from
various inputs and outputs.
* [Module](./doc/Module.md) - Super class for creating Kado modules.
* [Navigation](./doc/Navigation.md) - Create and manage application menus.
* [Parser](./doc/Parser.md) - Parse input strings to variables such as objects.
* [PathExp](./doc/PathExp.md) - Use path notation to validate routes on URIs.
* [Permission](./doc/Permission.md) - Create and test permission sets to
allow fine grained user control.
* [Profiler](./doc/Profiler.md) - Track application resource usage and
timing.
* [Router](./doc/Router.md) - Store and process application route points.
* [Search](./doc/Search.md) - Connect system for made for search.
* [Session](./doc/Session.md) - Session system for storing data against users.
* [TestRunner](./doc/TestRunner.md) - Define and run Test Suites and Tests.
* [Util](./doc/Util.md) - Misfit useful functions.
* [Validate](./doc/Validate.md) - Validate input.
* [View](./doc/View.md) - Connect system made for rendering.

## Questions or Problems?

Please see our [bug tracker](https://git.nullivex.com/kado/kado/issues)

## Change Log

Please see the [CHANGELOG.md](./CHANGELOG.md)

## Contributing

Please see the [Contribution Guidelines](./CONTRIBUTING.md)

## License
Kado Copyright (C) 2013-2020 Bryan Tong, NULLIVEX LLC. All rights reserved. Kado
is licensed under the Lesser GNU Public License version 3.0 or newer see
[LICENSE.md](./LICENSE.md) for a complete copy of applicable licenses.
